import { DialogService, Form, IGridEvent, LoadingService, Select, TextInput } from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class RedirecionamentoAssinaturaController {
    public setDadosFormParametrizacaoAssinatura() {
        LoadingService.show();
        Http.get('/getParamCentrAssinatura', {
        }).then((retorno) => {
            Metadata.getInstance<TextInput>('form-dados-assinatura-ID').value = retorno.data.data[0].ID;
            Metadata.getInstance<TextInput>('form-dados-assinatura-DSTOKENCERTSIGN').value = retorno.data.data[0].DSTOKENCERTSIGN;
            Metadata.getInstance<TextInput>('form-dados-assinatura-NMSENDERCERTSIGN').value = retorno.data.data[0].NMSENDERCERTSIGN;
            Metadata.getInstance<TextInput>('form-dados-assinatura-DSEMAILSENDERCERTSIGN').value = retorno.data.data[0].DSEMAILSENDERCERTSIGN;
            Metadata.getInstance<TextInput>('form-dados-assinatura-NRCPFSENDERCERTSIGN').value = retorno.data.data[0].NRCPFSENDERCERTSIGN;
            Metadata.getInstance<Select>('form-dados-assinatura-IDPADRAOASSINACERTSIGN').value = retorno.data.data[0].IDPADRAOASSINACERTSIGN;
            Metadata.getInstance<Select>('form-dados-assinatura-NRTIPODOCCERTSIGN').value = retorno.data.data[0].NRTIPODOCCERTSIGN;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public saveParametroAssinatura() {
        const form = Metadata.getInstance<Form>('form-dados-assinatura');
        const params = {
            ID: Metadata.getInstance<Select>('form-dados-assinatura-ID').value,
            DSTOKENCERTSIGN: Metadata.getInstance<Select>('form-dados-assinatura-DSTOKENCERTSIGN').value,
            NMSENDERCERTSIGN: Metadata.getInstance<Select>('form-dados-assinatura-NMSENDERCERTSIGN').value,
            DSEMAILSENDERCERTSIGN: Metadata.getInstance<Select>('form-dados-assinatura-DSEMAILSENDERCERTSIGN').value,
            NRCPFSENDERCERTSIGN: Metadata.getInstance<Select>('form-dados-assinatura-NRCPFSENDERCERTSIGN').value,
            IDPADRAOASSINACERTSIGN: Metadata.getInstance<Select>('form-dados-assinatura-IDPADRAOASSINACERTSIGN').value,
            NRTIPODOCCERTSIGN: Metadata.getInstance<Select>('form-dados-assinatura-NRTIPODOCCERTSIGN').value,
        };

        if (Utilities.validateForm(form)) {
            Http.post('/updateParamCentrAssinatura', params).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.showAlertCamposObrigatorios();
        }
    }

    public addNewRow({ component }: any) {
        const newId = 0;
        component.addNewRow({ ID: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`DSCHAVEASSINAT_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public saveRotasAssinatura({ event }: IGridEvent) {
        event!.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('tek-grid-rotas-assinatura');
        const rows = grid.getEditedRows();
        if (Utilities.validateGrid(grid)) {
            LoadingService.show();
            Http.post('/saveRotasAssinatura', rows).then(() => {
                grid.cancelEditedRows();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.showAlertCamposObrigatorios();
        }
    }

    public dialogDeleteRotaAssinatura({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'RedirecionamentoAssinaturaController.deleteRotaAssinatura');
    }

    public deleteRotaAssinatura() {
        DialogService.hide();
        const grid = Metadata.getInstance<TekGrid>('tek-grid-rotas-assinatura');
        LoadingService.show();
        Http.post('/deleteRotaAssinatura', grid.selectedRows).then(() => {
            grid.selectedRows = [];
            grid.cancelEditedRows();
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
